@@ -10,6 +10,16 @@ import { ICell, IJupyter } from './types';
 const Jupyter: IJupyter = (window as any)['Jupyter'];
 export const workRegx = /work\/lessons-(\d+)-(\w+)\.ipynb/;
 
+class BBYError extends Error {
+  constructor(message: string, cellIndex?: number, detail?: string) {
+    super(message);
+    //@ts-ignore
+    if (cellIndex) this.cellIndex = cellIndex;
+    //@ts-ignore
+    if (detail) this.detail = detail;
+  }
+}
+
 export const registerBbyButtons = () => {
   Jupyter.toolbar.add_buttons_group([
     Jupyter.keyboard_manager.actions.register(
@@ -408,11 +418,48 @@ const syncLesson = async function () {
 
   try {
     // 校验 jupyter 格式
-    parseStepsFromCells(content as any);
+    const steps = parseStepsFromCells(content as any);
+
+    // 检查是否预留verify块
+    for (let i = 0; i < steps.length; i++) {
+      const step = steps[i];
+      if (
+        (step as any).stepType === 'code' &&
+        step.verify === '' &&
+        !(step as any).verifyCode
+      ) {
+        let lastCodeCell =
+          step.codeAnswerCells[step.codeAnswerCells.length - 1];
+        if (
+          !lastCodeCell ||
+          lastCodeCell.$cellIndex! <
+            step.codeCells[step.codeCells.length - 1].$cellIndex!
+        ) {
+          lastCodeCell = step.codeCells[step.codeCells.length - 1];
+        }
+        if (!lastCodeCell) {
+          throw new BBYError(`代码题步骤 ${step.title} 的 Code cell 不合法`);
+        }
+        const nextCell = content.cells[(lastCodeCell.$cellIndex || 0) + 1];
+        const nextCellIsEmptyVerify =
+          nextCell &&
+          nextCell.cell_type === 'markdown' &&
+          nextCell.source === '';
+        if (!nextCellIsEmptyVerify) {
+          throw new BBYError(
+            `代码题步骤 ${step.title} 没有正确预留verify块`,
+            lastCodeCell.$cellIndex,
+            `cellIndex ${lastCodeCell.$cellIndex || 0}`
+          );
+        }
+      }
+    }
 
     // 清理垃圾 & 冗余数据
-    for await (const c of content.cells as ICell[]) {
-      // 清理运行记录
+    for (let cellIndex = 0; cellIndex < content.cells.length; cellIndex++) {
+      const c: ICell = content.cells[cellIndex];
+
+      // 清理运行数据
       if (c.outputs) {
         c.outputs = [];
         c.execution_count = null;
@@ -425,8 +472,28 @@ const syncLesson = async function () {
         }
       });
 
+      // 检查是否预留了视频cell
+      if (c.cell_type === 'markdown' && c.source.indexOf('### ') === 0) {
+        const nextIsDescCode =
+          content.cells[cellIndex + 1] &&
+          content.cells[cellIndex + 1].source.indexOf('#platform-desc') == 0;
+        if (!nextIsDescCode) {
+          throw new BBYError(
+            `步骤 ${c.source.replace('###', '')} 没有预留视频块`,
+            cellIndex,
+            `cellIndex: ${cellIndex}
+
+模板：
+#platform-desc
+# from ipyaliplayer import Player
+# Player(vid='替换视频ID', aspect_ratio=4/3)`
+          );
+        }
+      }
+
       // 解析 Python Quiz Cell，未来会向 JSON 配置方向迁移，抛弃 Python 写法
       const { type, code } = parseCodeCell(c.source);
+
       if (type === 'quiz') {
         const quizJSON = await parseQuizQuestions({
           code,
@@ -438,18 +505,18 @@ const syncLesson = async function () {
         // console.log('quiz 解析结果', { quizJSON });
 
         if (quizJSON?.statusCode) {
-          const quizError = new Error(`Quiz 解析失败: ${quizJSON?.message}`);
-          // eslint-disable-next-line @typescript-eslint/ban-ts-comment
-          // @ts-ignore
-          quizError.detail = code;
-          throw quizError;
+          throw new BBYError(
+            `Quiz 解析失败: ${quizJSON?.message}`,
+            cellIndex,
+            code
+          );
         }
 
         if (!quizJSON.value || !quizJSON.value.length) {
-          throw new Error('Quiz 解析失败: 缺少选项');
+          throw new BBYError('Quiz 解析失败: 缺少选项', cellIndex);
         }
 
-        for await (const question of quizJSON.value) {
+        for (const question of quizJSON.value) {
           // 如果题目来自题库，获取当前最新数据
           if (
             typeof question.id === 'string' &&
@@ -462,13 +529,19 @@ const syncLesson = async function () {
             });
             // console.log('来自题库的 quiz question', { questionJSON });
             if (!questionJSON.uuid) {
-              throw new Error(`Quiz 解析失败: 题目 ${question.id} 解析异常`);
+              throw new BBYError(
+                `Quiz 解析失败: 题目 ${question.id} 解析异常`,
+                cellIndex
+              );
             }
             const questionIndex = quizJSON.value.findIndex(
               (option: IQuestion) => option.id === question.id
             );
             if (questionIndex === -1) {
-              throw new Error(`Quiz 解析失败: 题目 ${question.id} 匹配失败`);
+              throw new BBYError(
+                `Quiz 解析失败: 题目 ${question.id} 匹配失败`,
+                cellIndex
+              );
             }
             // console.log('开始更新 quiz quetion', {
             //   questionWIP: quizJSON.value[questionIndex],
@@ -492,8 +565,6 @@ const syncLesson = async function () {
         }
       }
     }
-    // console.log({ cells: content.cells });
-
     // 同步
     _sync();
   } catch (error) {
@@ -504,10 +575,18 @@ const syncLesson = async function () {
         `<pre>${error?.detail || JSON.stringify(error)}</pre>`,
       sanitize: false,
       buttons: {
-        确认同步: {
+        强行同步: {
           click: _sync,
         },
-        取消同步: {},
+        [error.cellIndex != null ? '取消同步，查看错误Cell' : '取消同步']: {
+          class: 'btn-primary',
+          click: () => {
+            if (error.cellIndex != null) {
+              Jupyter.notebook.select(error.cellIndex);
+              Jupyter.notebook.focus_cell();
+            }
+          },
+        },
       },
     });
   }
